searchData={"content_type":"text/markdown","items":[{"doc":"Documentation for `Errata`.","ref":"Errata.html","title":"Errata","type":"module"},{"doc":"Support for creating custom domain error types, which can either be returned as error values or\nraised as exceptions.\n\nDomain errors are named, structured types that represent error conditions within a problem\ndomain or bounded context. Being named types means that errors have a meaningful name within a\nparticular context, and that name should be part of the Ubiquitous Language of the context.\nBeing structured types means that errors can have arbitrary contextual information attached to\nthem for logging or debugging purposes.\n\nDomain errors can be defined by creating an Elixir module that uses the `Errata.DomainError`\nmodule. Error types defined in this way are Elixir `Exception` structs with the following keys:\n\n  * `message` - human readable string describing the nature of the error\n  * `reason` - an atom describing the reason for the error, which can be used for pattern\n    matching or classifying the error\n  * `extra` - a map containing arbitrary contextual information or metadata about the error\n\nBecause these error types are defined with `defexception/1`, they can be raised as exceptions\nwith `raise/2`. However, because they represent domain errors rather than system or\ninfrastructure errors, in most cases it is more appropriate to create instances of the error\nstructs using `c:new/1` or `c:create/1` and use them as return values from domain functions,\neither directly or wrapped in an error tuple such as `{:error, my_domain_error}`.","ref":"Errata.DomainError.html","title":"Errata.DomainError","type":"behaviour"},{"doc":"To define a new custom domain error type, `use/2` the `Errata.DomainError` module in your own\nerror module:\n\n    defmodule MyApp.SomeError do\n      use Errata.DomainError,\n        default_message: \"something isn't right\"\n    end\n\nTo create instances of the error, to use as an error return value from a function, say, you can\nuse either `c:new/1` or `c:create/1`, passing params with extra information as desired. Note that\nif you use `c:create/1`, you must first `require` the error module, since this callback is\nimplemented as a macro. For example:\n\n    defmodule MyApp.SomeModule do\n      require MyApp.SomeError, as: SomeError\n\n      def some_function(arg) do\n        {:error, SomeError.create(reason: :helpful_tag, extra: %{arbitrary: \"metadata\", arg: arg})}\n      end\n    end\n\nTo raise errors as exceptions, simply use `raise/2` passing extra params as the second argument\nif desired:\n\n    defmodule MyApp.SomeModule do\n      require MyApp.SomeError, as: SomeError\n\n      def some_function!(arg) do\n        raise SomeError reason: :helpful_tag, extra: %{arbitrary: \"metadata\", arg: arg}\n      end\n    end","ref":"Errata.DomainError.html#module-usage","title":"Usage - Errata.DomainError","type":"behaviour"},{"doc":"Invoked to create a new instance of a domain error struct with default values and the current\n__ENV__.\n\nSee `c:create/1`.","ref":"Errata.DomainError.html#c:create/0","title":"Errata.DomainError.create/0","type":"macrocallback"},{"doc":"Invoked to create a new instance of a domain error struct with the given params and the current\n__ENV__.\n\nSince this is a  macro, the `__ENV__/0` special form is used to capture the `Macro.Env` struct\nfor the current environment and the public fields of this struct are placed in the exception\nstruct under the `:env` key. This provides access to information about the context in which the\nerror was created, such as the module, function, file, and line. See `t:env/0` for further\ndetails.\n\nNote that because this is a macro, callers must `require/2` the error module to be able to use it.","ref":"Errata.DomainError.html#c:create/1","title":"Errata.DomainError.create/1","type":"macrocallback"},{"doc":"Creates a new domain error struct of the given `error_type` using the given `params`.","ref":"Errata.DomainError.html#create/2","title":"Errata.DomainError.create/2","type":"function"},{"doc":"Creates a new domain error struct of the given `error_type` using the given `params` and `env`.","ref":"Errata.DomainError.html#create/3","title":"Errata.DomainError.create/3","type":"function"},{"doc":"Formats the given `domain_error` as a string.\n\nUses the `:message` field of the domain_error and combines the `:reason` field with it, if it is not nil.","ref":"Errata.DomainError.html#format_message/1","title":"Errata.DomainError.format_message/1","type":"function"},{"doc":"Returns `true` if `term` is a type of `Errata.DomainError`; otherwise returns `false`.\n\nAllowed in guard tests.","ref":"Errata.DomainError.html#is_domain_error/1","title":"Errata.DomainError.is_domain_error/1","type":"macro"},{"doc":"Invoked to create a new instance of a domain error struct with default values.\n\nSee `c:new/1`.","ref":"Errata.DomainError.html#c:new/0","title":"Errata.DomainError.new/0","type":"callback"},{"doc":"Invoked to create a new instance of a domain error struct with the given params.","ref":"Errata.DomainError.html#c:new/1","title":"Errata.DomainError.new/1","type":"callback"},{"doc":"","ref":"Errata.DomainError.html#to_json/2","title":"Errata.DomainError.to_json/2","type":"function"},{"doc":"","ref":"Errata.DomainError.html#to_map/1","title":"Errata.DomainError.to_map/1","type":"function"},{"doc":"Invoked to convert a domain error to a plain, JSON-compatible map.","ref":"Errata.DomainError.html#c:to_map/1","title":"Errata.DomainError.to_map/1","type":"callback"},{"doc":"Type to represent the `:env` field of domain error structs.\n\nThis struct is a subset of of `Macro.Env` and contains the following fields:\n\n  * `context` - the context of the environment; it can be nil (default context), :guard\n    (inside a guard) or :match (inside a match)\n  * `context_modules` - a list of modules defined in the current context\n  * `file` - the current absolute file name as a binary\n  * `function` - a tuple as {atom, integer}, where the first element is the function name and\n    the second its arity; returns nil if not inside a function\n  * `line` - the current line as an integer\n  * `module` - the current module name","ref":"Errata.DomainError.html#t:env/0","title":"Errata.DomainError.env/0","type":"type"},{"doc":"Type to represent allowable keys to use in params use for creating domain error structs.\n\nSee also `t:params/0`.","ref":"Errata.DomainError.html#t:param/0","title":"Errata.DomainError.param/0","type":"type"},{"doc":"Type to represent allowable values to be passes as params for creating domain error structs.\n\nThis effectively allows for using either a map or keyword list with allowable keys defined by\n`t:param/0`.","ref":"Errata.DomainError.html#t:params/0","title":"Errata.DomainError.params/0","type":"type"},{"doc":"Type to represent domain error structs as defined by this module.\n\nError structs are `Exception` structs that have additional fields to contain extra contextual\ninformation, such as an error reason or details about the context in which the error occurred.","ref":"Errata.DomainError.html#t:t/0","title":"Errata.DomainError.t/0","type":"type"},{"doc":"# Changelog\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](http://keepachangelog.com/en/1.0.0/)\nand this project adheres to [Semantic Versioning](http://semver.org/spec/v2.0.0.html).","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html","title":"License","type":"extras"}]}